@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2009-2010 the bpython authors.
+# Copyright (c) 2009-2011 the bpython authors.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,17 +32,19 @@ import inspect
 import optparse
 import os
 import sys
-from locale import LC_ALL, getpreferredencoding, setlocale
+from locale import getpreferredencoding
 
 import gobject
 import gtk
 import pango
 from pygments.lexers import PythonLexer
 
-from bpython import importcompletion, repl
+from bpython import importcompletion, repl, translations
 from bpython.formatter import theme_map
+from bpython.translations import _
 import bpython.args
 
+
 py3 = sys.version_info[0] == 3
 
 _COLORS = dict(b='blue', c='cyan', g='green', m='magenta', r='red',
@@ -81,18 +83,20 @@ class ArgspecFormatter(object):
         return string
 
     def formatvalue(self, value):
-        return '=%s' % (value, )
+        return '=%r' % (value, )
 
 
 class ExceptionDialog(gtk.MessageDialog):
-    def __init__(self, exc_type, exc_value, tb, text='An error occurred.'):
+    def __init__(self, exc_type, exc_value, tb, text=None):
+        if text is None:
+            text = _('An error occurred.')
         gtk.MessageDialog.__init__(self, buttons=gtk.BUTTONS_CLOSE,
                                    type=gtk.MESSAGE_ERROR,
                                    message_format=text)
         self.set_resizable(True)
         import cgitb
         text = cgitb.text((exc_type, exc_value, tb), 5)
-        expander = gtk.Expander('Exception details')
+        expander = gtk.Expander(_('Exception details'))
         self.vbox.pack_start(expander)
         textview = gtk.TextView()
         textview.get_buffer().set_text(text)
@@ -108,9 +112,9 @@ class ExceptionManager(object):
     the exception's type, value, a traceback and a text to display as
     arguments.
     """
-    def __init__(self, DialogType, text='An error occurred.'):
+    def __init__(self, DialogType, text=None):
         self.DialogType = DialogType
-        self.text = text
+        self.text = text or _('An error occurred.')
 
     def __enter__(self):
         return self
@@ -144,7 +148,7 @@ class Statusbar(gtk.Statusbar):
     """Contains feedback messages"""
     def __init__(self):
         gtk.Statusbar.__init__(self)
-        self.context_id = self.get_context_id('Statusbar')
+        self.context_id = self.get_context_id(_('Statusbar'))
 
     def message(self, s, n=3):
         self.clear()
@@ -221,7 +225,7 @@ class SuggestionWindow(gtk.Window):
         width, height = self.get_size()
         self.style.paint_flat_box(self.window, gtk.STATE_NORMAL,
                                   gtk.SHADOW_OUT, None, self,
-                                  'tooltip', 0, 0, width, height)
+                                  _('tooltip'), 0, 0, width, height)
         gtk.Window.do_expose_event(self, event)
 
     def forward(self):
@@ -290,7 +294,7 @@ class GTKInteraction(repl.Interaction):
         return response == gtk.RESPONSE_YES
 
     def file_prompt(self, s):
-        chooser = gtk.FileChooserDialog(title="File to save to",
+        chooser = gtk.FileChooserDialog(title=_("File to save to"),
                                         action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                         buttons=(gtk.STOCK_CANCEL,
                                                  gtk.RESPONSE_CANCEL,
@@ -301,12 +305,12 @@ class GTKInteraction(repl.Interaction):
         chooser.set_current_folder(os.path.expanduser('~'))
 
         pyfilter = gtk.FileFilter()
-        pyfilter.set_name("Python files")
+        pyfilter.set_name(_("Python files"))
         pyfilter.add_pattern("*.py")
         chooser.add_filter(pyfilter)
 
         allfilter = gtk.FileFilter()
-        allfilter.set_name("All files")
+        allfilter.set_name(_("All files"))
         allfilter.add_pattern("*")
         chooser.add_filter(allfilter)
 
@@ -477,7 +481,7 @@ class ReplWidget(gtk.TextView, repl.Repl):
                     show_source_in_new_window(source, self.config.color_gtk_scheme,
                                               self.config.syntax)
                 else:
-                    self.interact.notify('Cannot show source.')
+                    self.interact.notify(_('Cannot show source.'))
             elif event.keyval == gtk.keysyms.Return:
                 if self.list_win_visible:
                     self.list_win_visible = False
@@ -668,13 +672,13 @@ class ReplWidget(gtk.TextView, repl.Repl):
         Show the appropriate Python prompt.
         """
         if more:
-            text = '... '
+            text = self.ps2
         else:
-            text = '>>> '
+            text = self.ps1
         with self.editing:
             iter_ = self.get_cursor_iter()
             self.text_buffer.insert_with_tags_by_name(iter_, text, 'prompt')
-        iter_.forward_chars(4)
+        iter_.forward_chars(len(text))
         mark = self.text_buffer.create_mark('line_start', iter_, True)
         self.text_buffer.place_cursor(iter_)
         self.scroll_to_mark(mark, 0.2)
@@ -684,7 +688,6 @@ class ReplWidget(gtk.TextView, repl.Repl):
         # Save mark for easy referencing later
         self.text_buffer.create_mark('line%i_start' % (len(self.buffer), ),
                                      self.get_line_start_iter(), True)
-        self.rl_history.append(line)
         iter_ = self.get_line_end_iter()
         self.text_buffer.place_cursor(iter_)
         with self.editing:
@@ -763,13 +766,12 @@ def init_import_completion():
 
 
 def main(args=None):
+    translations.init()
 
-    setlocale(LC_ALL, '')
-
-    gtk_options = ('gtk-specific options',
-                   "Options specific to bpython's Gtk+ front end",
+    gtk_options = (_('gtk-specific options'),
+                   _("Options specific to bpython's Gtk+ front end"),
                    [optparse.Option('--socket-id', dest='socket_id',
-                                    type='int', help='Embed bpython')])
+                                    type='int', help=_('Embed bpython'))])
     config, options, exec_args = bpython.args.parse(args, gtk_options,
                                                     True)
 
@@ -825,7 +827,7 @@ def main(args=None):
     pastebin.connect("activate", repl_widget.do_paste)
     filemenu.append(pastebin)
 
-    pastebin_partial = gtk.MenuItem("Pastebin selection")
+    pastebin_partial = gtk.MenuItem(_("Pastebin selection"))
     pastebin_partial.connect("activate", repl_widget.do_partial_paste)
     filemenu.append(pastebin_partial)
 
@@ -856,10 +858,7 @@ def main(args=None):
         gtk.main()
     except KeyboardInterrupt:
         pass
-    finally:
-        if config.hist_length:
-            histfilename = os.path.expanduser(config.hist_file)
-            repl_widget.rl_history.save(histfilename, getpreferredencoding())
+
     return 0
 
 if __name__ == '__main__':
