@@ -146,17 +146,25 @@ class Interpreter(code.InteractiveInterpreter):
 
 class History(object):
 
-    def __init__(self, entries=None):
+    def __init__(self, entries=None, duplicates=False):
         if entries is None:
             self.entries = ['']
         else:
             self.entries = list(entries)
         self.index = 0
         self.saved_line = ''
+        self.duplicates = duplicates
 
     def append(self, line):
         line = line.rstrip('\n')
         if line:
+            if not self.duplicates:
+                # remove duplicates
+                try:
+                    while True:
+                        self.entries.remove(line)
+                except ValueError:
+                    pass
             self.entries.append(line)
 
     def first(self):
@@ -165,21 +173,62 @@ class History(object):
             self.index = len(self.entries)
         return self.entries[-self.index]
 
-    def back(self):
+    def back(self, start=True, search=False):
         """Move one step back in the history."""
         if not self.is_at_end:
-            self.index += 1
-        return self.entries[-self.index]
+            if search:
+                self.index += self.find_partial_match_backward(self.saved_line)
+            elif start:
+                self.index += self.find_match_backward(self.saved_line)
+            else:
+                self.index += 1
+        return self.entries[-self.index] if self.index else self.saved_line
 
-    def forward(self):
+    def find_match_backward(self, search_term):
+        filtered_list_len = len(self.entries) - self.index
+        for idx, val in enumerate(reversed(self.entries[:filtered_list_len])):
+            if val.startswith(search_term):
+                return idx + 1
+        return 0
+
+    def find_partial_match_backward(self, search_term):
+        filtered_list_len = len(self.entries) - self.index
+        for idx, val in enumerate(reversed(self.entries[:filtered_list_len])):
+            if search_term in val:
+                return idx + 1
+        return 0
+
+
+    def forward(self, start=True, search=False):
         """Move one step forward in the history."""
         if self.index > 1:
-            self.index -= 1
-            return self.entries[-self.index]
+            if search:
+                self.index -= self.find_partial_match_forward(self.saved_line)
+            elif start:
+                self.index -= self.find_match_forward(self.saved_line)
+            else:
+                self.index -= 1
+            return self.entries[-self.index] if self.index else self.saved_line
         else:
             self.index = 0
             return self.saved_line
 
+    def find_match_forward(self, search_term):
+        filtered_list_len = len(self.entries) - self.index + 1
+        for idx, val in enumerate(self.entries[filtered_list_len:]):
+            if val.startswith(search_term):
+                return idx + 1
+        return self.index
+
+    def find_partial_match_forward(self, search_term):
+        filtered_list_len = len(self.entries) - self.index + 1
+        for idx, val in enumerate(self.entries[filtered_list_len:]):
+            if search_term in val:
+                return idx + 1
+        return self.index
+
+
+
     def last(self):
         """Move forward to the end of the history."""
         if not self.is_at_start:
@@ -320,7 +369,7 @@ class Repl(object):
         self.interp = interp
         self.interp.syntaxerror_callback = self.clear_current_line
         self.match = False
-        self.rl_history = History()
+        self.rl_history = History(duplicates=config.hist_duplicates)
         self.s_hist = []
         self.history = []
         self.evaluating = False
@@ -352,7 +401,8 @@ class Repl(object):
 
         pythonhist = os.path.expanduser(self.config.hist_file)
         if os.path.exists(pythonhist):
-            self.rl_history.load(pythonhist, getpreferredencoding())
+            self.rl_history.load(pythonhist,
+                    getpreferredencoding() or "ascii")
 
     def startup(self):
         """
@@ -597,6 +647,11 @@ class Repl(object):
                 e = True
             else:
                 matches = self.completer.matches
+                if (self.config.complete_magic_methods and self.buffer and
+                    self.buffer[0].startswith("class ") and
+                    self.current_line().lstrip().startswith("def ")):
+                    matches.extend(name for name in self.config.magic_methods
+                                   if name.startswith(cw))
 
         if not e and self.argspec:
             matches.extend(name + '=' for name in self.argspec[1][0]
@@ -715,8 +770,16 @@ class Repl(object):
             not self.interact.confirm("Pastebin buffer? (y/N) ")):
             self.interact.notify("Pastebin aborted")
             return
+        return self.do_pastebin(s)
 
-        pasteservice = ServerProxy(self.config.pastebin_url)
+    def do_pastebin(self, s):
+        """Actually perform the upload."""
+        try:
+            pasteservice = ServerProxy(self.config.pastebin_url)
+        except IOError, e:
+            self.interact.notify("Pastebin error for URL '%s': %s" %
+                                 (self.config.pastebin_url, str(e)))
+            return
 
         if s == self.prev_pastebin_content:
             self.interact.notify('Duplicate pastebin. Previous URL: ' +
