@@ -1,7 +1,8 @@
+import os
 import unittest
 from itertools import islice
 
-from bpython import repl
+from bpython import config, repl
 
 
 class TestHistory(unittest.TestCase):
@@ -129,6 +130,71 @@ class TestMatchesIterator(unittest.TestCase):
         self.assertNotEqual(list(slice), self.matches)
         self.assertEqual(list(newslice), newmatches)
 
+class TestRepl(repl.Repl):
+    def __init__(self):
+        config_struct = config.Struct()
+        config.loadini(config_struct, os.devnull)
+        repl.Repl.__init__(self, repl.Interpreter(), config_struct)
+        self.input_line = ""
+
+    def current_line(self):
+        return self.input_line
+
+class TestArgspec(unittest.TestCase):
+    def setUp(self):
+        self.repl = TestRepl()
+        self.repl.push("def spam(a, b, c):\n", False)
+        self.repl.push("    pass\n", False)
+        self.repl.push("\n", False)
+
+    def setInputLine(self, line):
+        """Set current input line of the test REPL."""
+        self.repl.input_line = line
+
+    def test_func_name(self):
+        for (line, expected_name) in [("spam(", "spam"),
+                                      ("spam(map([]", "map"),
+                                      ("spam((), ", "spam")]:
+            self.setInputLine(line)
+            self.assertTrue(self.repl.get_args())
+            self.assertEqual(self.repl.current_func.__name__, expected_name)
+
+    def test_syntax_error_parens(self):
+        for line in ["spam(]", "spam([)", "spam())"]:
+            self.setInputLine(line)
+            # Should not explode
+            self.repl.get_args()
+
+    def test_kw_arg_position(self):
+        self.setInputLine("spam(a=0")
+        self.assertTrue(self.repl.get_args())
+        self.assertEqual(self.repl.argspec[3], "a")
+
+        self.setInputLine("spam(1, b=1")
+        self.assertTrue(self.repl.get_args())
+        self.assertEqual(self.repl.argspec[3], "b")
+
+        self.setInputLine("spam(1, c=2")
+        self.assertTrue(self.repl.get_args())
+        self.assertEqual(self.repl.argspec[3], "c")
+
+    def test_lambda_position(self):
+        self.setInputLine("spam(lambda a, b: 1, ")
+        self.assertTrue(self.repl.get_args())
+        self.assertTrue(self.repl.argspec)
+        # Argument position
+        self.assertEqual(self.repl.argspec[3], 1)
+
+    def test_name_in_assignment_without_spaces(self):
+        # Issue #127
+        self.setInputLine("x=range(")
+        self.assertTrue(self.repl.get_args())
+        self.assertEqual(self.repl.current_func.__name__, "range")
+
+    def test_nonexistent_name(self):
+        self.setInputLine("spamspamspam(")
+        self.assertFalse(self.repl.get_args())
+
 
 if __name__ == '__main__':
     unittest.main()
