@@ -37,6 +37,7 @@ from __future__ import absolute_import, with_statement, division
 
 import sys
 import os
+import time
 import locale
 import signal
 from types import ModuleType
@@ -112,37 +113,62 @@ else:
             return EvalProtocol(self.repl)
 
 
-class TwistedEventLoop(urwid.TwistedEventLoop):
+if urwid.VERSION < (1, 0, 0):
+    class TwistedEventLoop(urwid.TwistedEventLoop):
 
-    """TwistedEventLoop modified to properly stop the reactor.
+        """TwistedEventLoop modified to properly stop the reactor.
 
-    urwid 0.9.9 and 0.9.9.1 crash the reactor on ExitMainLoop instead
-    of stopping it. One obvious way this breaks is if anything used
-    the reactor's thread pool: that thread pool is not shut down if
-    the reactor is not stopped, which means python hangs on exit
-    (joining the non-daemon threadpool threads that never exit). And
-    the default resolver is the ThreadedResolver, so if we looked up
-    any names we hang on exit. That is bad enough that we hack up
-    urwid a bit here to exit properly.
-    """
+        urwid 0.9.9 and 0.9.9.1 crash the reactor on ExitMainLoop instead
+        of stopping it. One obvious way this breaks is if anything used
+        the reactor's thread pool: that thread pool is not shut down if
+        the reactor is not stopped, which means python hangs on exit
+        (joining the non-daemon threadpool threads that never exit). And
+        the default resolver is the ThreadedResolver, so if we looked up
+        any names we hang on exit. That is bad enough that we hack up
+        urwid a bit here to exit properly.
+        """
 
-    def handle_exit(self, f):
-        def wrapper(*args, **kwargs):
-            try:
-                return f(*args, **kwargs)
-            except urwid.ExitMainLoop:
-                # This is our change.
-                self.reactor.stop()
-            except:
-                # This is the same as in urwid.
-                # We are obviously not supposed to ever hit this.
-                import sys
-                print sys.exc_info()
-                self._exc_info = sys.exc_info()
-                self.reactor.crash()
-        return wrapper
+        def handle_exit(self, f):
+            def wrapper(*args, **kwargs):
+                try:
+                    return f(*args, **kwargs)
+                except urwid.ExitMainLoop:
+                    # This is our change.
+                    self.reactor.stop()
+                except:
+                    # This is the same as in urwid.
+                    # We are obviously not supposed to ever hit this.
+                    import sys
+                    print sys.exc_info()
+                    self._exc_info = sys.exc_info()
+                    self.reactor.crash()
+            return wrapper
+else:
+    TwistedEventLoop = urwid.TwistedEventLoop
 
 
+class StatusbarEdit(urwid.Edit):
+    """Wrapper around urwid.Edit used for the prompt in Statusbar.
+
+    This class only adds a single signal that is emitted if the user presses
+    Enter."""
+
+    signals = urwid.Edit.signals + ['prompt_enter']
+
+    def __init__(self, *args, **kwargs):
+        self.single = False
+        urwid.Edit.__init__(self, *args, **kwargs)
+
+    def keypress(self, size, key):
+        if self.single:
+            urwid.emit_signal(self, 'prompt_enter', self, key)
+        elif key == 'enter':
+            urwid.emit_signal(self, 'prompt_enter', self, self.get_edit_text())
+        else:
+            return urwid.Edit.keypress(self, size, key)
+
+urwid.register_signal(StatusbarEdit, 'prompt_enter')
+
 class Statusbar(object):
 
     """Statusbar object, ripped off from bpython.cli.
@@ -163,12 +189,88 @@ class Statusbar(object):
     The "widget" attribute is an urwid widget.
     """
 
-    def __init__(self, config, s=None):
+    signals = ['prompt_result']
+
+    def __init__(self, config, s=None, main_loop=None):
         self.config = config
+        self.timer = None
+        self.main_loop = main_loop
         self.s = s or ''
 
-        # XXX wrap in AttrMap for wrapping?
-        self.widget = urwid.Text(('main', self.s))
+        self.text = urwid.Text(('main', self.s))
+        # use wrap mode 'clip' to just cut off at the end of line
+        self.text.set_wrap_mode('clip')
+
+        self.edit = StatusbarEdit(('main', ''))
+        urwid.connect_signal(self.edit, 'prompt_enter', self._on_prompt_enter)
+
+        self.widget = urwid.Columns([self.text, self.edit])
+
+    def _check(self, callback, userdata=None):
+        """This is the method is called from the timer to reset the status bar."""
+        self.timer = None
+        self.settext(self.s)
+
+    def message(self, s, n=3):
+        """Display a message for a short n seconds on the statusbar and return
+        it to its original state."""
+
+        self.settext(s)
+        self.timer = self.main_loop.set_alarm_in(n, self._check)
+
+    def _reset_timer(self):
+        """Reset the timer from message."""
+        if self.timer is not None:
+            self.main_loop.remove_alarm(self.timer)
+            self.timer = None
+
+    def prompt(self, s=None, single=False):
+        """Prompt the user for some input (with the optional prompt 's'). After
+        the user hit enter the signal 'prompt_result' will be emited and the
+        status bar will be reset. If single is True, the first keypress will be
+        returned."""
+
+        self._reset_timer()
+
+        self.edit.single = single
+        self.edit.set_caption(('main', s or '?'))
+        self.edit.set_edit_text('')
+        # hide the text and display the edit widget
+        if not self.edit in self.widget.widget_list:
+            self.widget.widget_list.append(self.edit)
+        if self.text in self.widget.widget_list:
+            self.widget.widget_list.remove(self.text)
+        self.widget.set_focus_column(0)
+
+    def settext(self, s, permanent=False):
+        """Set the text on the status bar to a new value. If permanent is True,
+        the new value will be permanent. If that status bar is in prompt mode,
+        the prompt will be aborted. """
+
+        self._reset_timer()
+
+        # hide the edit and display the text widget
+        if self.edit in self.widget.widget_list:
+            self.widget.widget_list.remove(self.edit)
+        if not self.text in self.widget.widget_list:
+            self.widget.widget_list.append(self.text)
+
+        self.text.set_text(('main', s))
+        if permanent:
+          self.s = s
+
+    def clear(self):
+        """Clear the status bar."""
+        self.settext('')
+
+    def _on_prompt_enter(self, edit, new_text):
+        """Reset the statusbar and pass the input from the prompt to the caller
+        via 'prompt_result'."""
+        self.settext(self.s)
+        urwid.emit_signal(self, 'prompt_result', new_text)
+
+urwid.register_signal(Statusbar, 'prompt_result')
+
 
 def decoding_input_filter(keys, raw):
     """Input filter for urwid which decodes each key with the locale's
@@ -225,7 +327,7 @@ class BPythonEdit(urwid.Edit):
         self._bpy_selectable = True
         self._bpy_may_move_cursor = False
         self.config = config
-        self.tab_length = config.tab_length 
+        self.tab_length = config.tab_length
         urwid.Edit.__init__(self, *args, **kwargs)
 
     def set_edit_pos(self, pos):
@@ -251,7 +353,11 @@ class BPythonEdit(urwid.Edit):
 
         You should arrange for this to be called from the 'change' signal.
         """
-        self._bpy_text, self._bpy_attr = urwid.decompose_tagmarkup(markup)
+        if markup:
+            self._bpy_text, self._bpy_attr = urwid.decompose_tagmarkup(markup)
+        else:
+            # decompose_tagmarkup in some urwids fails on the empty list
+            self._bpy_text, self._bpy_attr = '', []
         # This is redundant when we're called off the 'change' signal.
         # I'm assuming this is cheap, making that ok.
         self._invalidate()
@@ -305,9 +411,6 @@ class BPythonEdit(urwid.Edit):
                     self.set_edit_text(line[:-self.tab_length])
                 else:
                     return urwid.Edit.keypress(self, size, key)
-            elif key == 'pastebin':
-                # do pastebin
-                pass
             else:
                 # TODO: Add in specific keypress fetching code here
                 return urwid.Edit.keypress(self, size, key)
@@ -413,44 +516,64 @@ class Tooltip(urwid.BoxWidget):
         return canvas
 
 class URWIDInteraction(repl.Interaction):
-    def __init__(self, config, statusbar=None):
+    def __init__(self, config, statusbar, frame):
         repl.Interaction.__init__(self, config, statusbar)
+        self.frame = frame
+        urwid.connect_signal(statusbar, 'prompt_result', self._prompt_result)
+        self.callback = None
 
-    def confirm(self, q):
-        """Ask for yes or no and return boolean"""
-        try:
-            reply = self.statusbar.prompt(q)
-        except ValueError:
-            return False
+    def confirm(self, q, callback):
+        """Ask for yes or no and call callback to return the result"""
 
-        return reply.lower() in (_('y'), _('yes'))
+        def callback_wrapper(result):
+            callback(result.lower() in (_('y'), _('yes')))
+
+        self.prompt(q, callback_wrapper, single=True)
 
     def notify(self, s, n=10):
         return self.statusbar.message(s, n)
 
+    def prompt(self, s, callback=None, single=False):
+        """Prompt the user for input. The result will be returned via calling
+        callback. Note that there can only be one prompt active. But the
+        callback can already start a new prompt."""
+
+        if self.callback is not None:
+            raise Exception('Prompt already in progress')
+
+        self.callback = callback
+        self.statusbar.prompt(s, single=single)
+        self.frame.set_focus('footer')
+
+    def _prompt_result(self, text):
+        self.frame.set_focus('body')
+        if self.callback is not None:
+            # The callback might want to start another prompt, so reset it
+            # before calling the callback.
+            callback = self.callback
+            self.callback = None
+            callback(text)
+
 
 class URWIDRepl(repl.Repl):
 
+    _time_between_redraws = .05 # seconds
+
     def __init__(self, event_loop, palette, interpreter, config):
         repl.Repl.__init__(self, interpreter, config)
 
+        self._redraw_handle = None
+        self._redraw_pending = False
+        self._redraw_time = 0
+
         self.listbox = BPythonListBox(urwid.SimpleListWalker([]))
 
-        # String is straight from bpython.cli
-        self.statusbar = Statusbar(config,
-            _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
-              " <%s> Pager  <%s> Show Source ") %
-              (config.undo_key, config.save_key, config.pastebin_key,
-               config.last_output_key, config.show_source_key))
-
-        self.interact = URWIDInteraction(self.config, self.statusbar)
-
         self.tooltip = urwid.ListBox(urwid.SimpleListWalker([]))
         self.tooltip.grid = None
         self.overlay = Tooltip(self.listbox, self.tooltip)
         self.stdout_hist = ''
 
-        self.frame = urwid.Frame(self.overlay, footer=self.statusbar.widget)
+        self.frame = urwid.Frame(self.overlay)
 
         if urwid.get_encoding_mode() == 'narrow':
             input_filter = decoding_input_filter
@@ -463,6 +586,15 @@ class URWIDRepl(repl.Repl):
             event_loop=event_loop, unhandled_input=self.handle_input,
             input_filter=input_filter, handle_mouse=False)
 
+        # String is straight from bpython.cli
+        self.statusbar = Statusbar(config,
+            _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
+              " <%s> Pager  <%s> Show Source ") %
+              (config.undo_key, config.save_key, config.pastebin_key,
+               config.last_output_key, config.show_source_key), self.main_loop)
+        self.frame.set_footer(self.statusbar.widget)
+        self.interact = URWIDInteraction(self.config, self.statusbar, self.frame)
+
         self.edits = []
         self.edit = None
         self.current_output = None
@@ -478,6 +610,13 @@ class URWIDRepl(repl.Repl):
                 self.current_output = urwid.Text(('output', s))
                 if self.edit is None:
                     self.listbox.body.append(self.current_output)
+                    # Focus the widget we just added to force the
+                    # listbox to scroll. This causes output to scroll
+                    # if the user runs a blocking call that prints
+                    # more than a screenful, instead of staying
+                    # scrolled to the previous input line and then
+                    # jumping to the bottom when done.
+                    self.listbox.set_focus(len(self.listbox.body) - 1)
                 else:
                     self.listbox.body.insert(-1, self.current_output)
                     # The edit widget should be focused and *stay* focused.
@@ -489,9 +628,37 @@ class URWIDRepl(repl.Repl):
                     ('output', self.current_output.text + s))
         if orig_s.endswith('\n'):
             self.current_output = None
-        # TODO: maybe do the redraw after a short delay
-        # (for performance)
-        self.main_loop.draw_screen()
+
+        # If we hit this repeatedly in a loop the redraw is rather
+        # slow (testcase: pprint(__builtins__). So if we have recently
+        # drawn the screen already schedule a call in the future.
+        #
+        # Unfortunately we may hit this function repeatedly through a
+        # blocking call triggered by the user, in which case our
+        # timeout will not run timely as we do not return to urwid's
+        # eventloop. So we manually check if our timeout has long
+        # since expired, and redraw synchronously if it has.
+        if self._redraw_handle is None:
+            self.main_loop.draw_screen()
+
+            def maybe_redraw(loop, self):
+                if self._redraw_pending:
+                    loop.draw_screen()
+                    self._redraw_pending = False
+
+                self._redraw_handle = None
+
+            self._redraw_handle = self.main_loop.set_alarm_in(
+                self._time_between_redraws, maybe_redraw, self)
+            self._redraw_time = time.time()
+        else:
+            self._redraw_pending = True
+            now = time.time()
+            if now - self._redraw_time > 2 * self._time_between_redraws:
+                # The timeout is well past expired, assume we're
+                # blocked and redraw synchronously.
+                self.main_loop.draw_screen()
+                self._redraw_time = now
 
     def current_line(self):
         """Return the current line (the one the cursor is in)."""
@@ -513,7 +680,7 @@ class URWIDRepl(repl.Repl):
         # Stolen from cli. TODO: clean up and split out.
         if (not text or
             (not text[-1].isalnum() and text[-1] not in ('.', '_'))):
-             return
+            return
 
         # Seek backwards in text for the first non-identifier char:
         for i, c in enumerate(reversed(text)):
@@ -740,8 +907,20 @@ class URWIDRepl(repl.Repl):
         self.prompt(False)
 
     def keyboard_interrupt(self):
-        # Do we need to do more here? Break out of multiline input perhaps?
-        self.echo('KeyboardInterrupt')
+        # If the user is currently editing, interrupt him. This
+        # mirrors what the regular python REPL does.
+        if self.edit is not None:
+            # XXX this is a lot of code, and I am not sure it is
+            # actually enough code. Needs some testing.
+            self.edit.make_readonly()
+            self.edit = None
+            self.buffer = []
+            self.echo('KeyboardInterrupt')
+            self.prompt(False)
+        else:
+            # I do not quite remember if this is reachable, but let's
+            # be safe.
+            self.echo('KeyboardInterrupt')
 
     def prompt(self, more):
         # Clear current output here, or output resulting from the
@@ -751,14 +930,20 @@ class URWIDRepl(repl.Repl):
         # XXX is this the right place?
         self.rl_history.reset()
         # XXX what is s_hist?
+
+        # We need the caption to use unicode as urwid normalizes later
+        # input to be the same type, using ascii as encoding. If the
+        # caption is bytes this breaks typing non-ascii into bpython.
+        # Currently this decodes using ascii as I do not know where
+        # ps1 is getting loaded from. If anyone wants to make
+        # non-ascii prompts work feel free to fix this.
         if not more:
-            self.edit = BPythonEdit(self.config,
-                                    caption=('prompt', self.ps1))
+            caption = ('prompt', self.ps1.decode('ascii'))
             self.stdout_hist += self.ps1
         else:
-            self.edit = BPythonEdit(self.config,
-                                    caption=('prompt_more', self.ps2))
+            caption = ('prompt_more', self.ps2.decode('ascii'))
             self.stdout_hist += self.ps2
+        self.edit = BPythonEdit(self.config, caption=caption)
 
         urwid.connect_signal(self.edit, 'change', self.on_input_change)
         urwid.connect_signal(self.edit, 'edit-pos-changed',
@@ -791,6 +976,11 @@ class URWIDRepl(repl.Repl):
         edit.set_edit_markup(list(format_tokens(tokens)))
 
     def handle_input(self, event):
+        # Since most of the input handling here should be handled in the edit
+        # instead, we return here early if the edit doesn't have the focus.
+        if self.frame.get_focus() != 'body':
+            return
+
         if event == 'enter':
             inp = self.edit.get_edit_text()
             self.history.append(inp)
@@ -904,10 +1094,14 @@ def main(args=None, locals_=None, banner=None):
                 ]))
 
     if options.help_reactors:
-        from twisted.application import reactors
-        # Stolen from twisted.application.app (twistd).
-        for r in reactors.getReactorTypes():
-            print '    %-4s\t%s' % (r.shortName, r.description)
+        try:
+            from twisted.application import reactors
+            # Stolen from twisted.application.app (twistd).
+            for r in reactors.getReactorTypes():
+                print '    %-4s\t%s' % (r.shortName, r.description)
+        except ImportError:
+            sys.stderr.write('No reactors are available. Please install '
+                'twisted for reactor support.\n')
         return
 
     palette = [
@@ -922,7 +1116,12 @@ def main(args=None, locals_=None, banner=None):
         options.reactor = 'select'
 
     if options.reactor:
-        from twisted.application import reactors
+        try:
+            from twisted.application import reactors
+        except ImportError:
+            sys.stderr.write('No reactors are available. Please install '
+                'twisted for reactor support.\n')
+            return
         try:
             # XXX why does this not just return the reactor it installed?
             reactor = reactors.installReactor(options.reactor)
@@ -945,8 +1144,14 @@ def main(args=None, locals_=None, banner=None):
         locals_ = main_mod.__dict__
 
     if options.plugin:
-        from twisted import plugin
-        from twisted.application import service
+        try:
+            from twisted import plugin
+            from twisted.application import service
+        except ImportError:
+            sys.stderr.write('No twisted plugins are available. Please install '
+                'twisted for twisted plugin support.\n')
+            return
+
         for plug in plugin.getPlugins(service.IServiceMaker):
             if plug.tapname == options.plugin:
                 break
@@ -1066,6 +1271,7 @@ def load_urwid_command_map(config):
     urwid.command_map[key_dispatch[config.down_one_line_key]] = 'cursor down'
     urwid.command_map[key_dispatch['C-a']] = 'cursor max left'
     urwid.command_map[key_dispatch['C-e']] = 'cursor max right'
+    urwid.command_map[key_dispatch[config.pastebin_key]] = 'pastebin'
 """
             'clear_line': 'C-u',
             'clear_screen': 'C-l',
